@@ -1,7 +1,7 @@
 # reportbug_submit module - email and GnuPG functions
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2006 Chris Lawrence
-#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2023 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -176,9 +176,9 @@ def mime_attach(body, attachments, charset, body_charset=None):
         try:
             fp = open(attachment)
             fp.close()
-        except EnvironmentError as x:
+        except EnvironmentError as ex:
             ewrite("Warning: opening '%s' failed: %s.\n", attachment,
-                   x.strerror)
+                   ex.strerror)
             failed = True
             continue
         ctype = None
@@ -423,13 +423,16 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         jalist = ' '.join(malist)
 
         faddr = email.utils.parseaddr(fromaddr)[1]
-        if envelopefrom:
+        if envelopefrom is not None:
             envfrom = email.utils.parseaddr(envelopefrom)[1]
         else:
             envfrom = faddr
         ewrite("Sending message via %s...\n", mta)
-        pipe = os.popen('%s -f %s -oi -oem %s' % (
-            mta, shlex.quote(envfrom), jalist), 'w')
+        if envfrom == "":
+            pipe = os.popen(f'{mta} -oi -oem {jalist}', 'w')
+        else:
+            pipe = os.popen('%s -f %s -oi -oem %s' % (
+                mta, shlex.quote(envfrom), jalist), 'w')
         using_sendmail = True
 
     # saving a backup of the report
@@ -440,6 +443,12 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
     if smtphost:
         toaddrs = [x[1] for x in alist]
 
+        try:
+            smtphost, smtpport = smtphost.split(":")
+            smtpport = int(smtpport)
+        except ValueError:
+            smtpport = 0
+
         tryagain = True
         refused = None
         retry = 0
@@ -452,12 +461,12 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                 # submit
                 if smtphost.lower() == 'reportbug.debian.org':
                     conn = smtplib.SMTP(smtphost, 587)
-                elif smtphost.endswith(':465'):
+                elif smtpport == 465:
                     # ignore smtptls setting since port 465 implies SSL
                     smtptls = None
-                    conn = smtplib.SMTP_SSL(smtphost)
+                    conn = smtplib.SMTP_SSL(smtphost, 465)
                 else:
-                    conn = smtplib.SMTP(smtphost)
+                    conn = smtplib.SMTP(smtphost, smtpport)
                 response = conn.ehlo()
                 if not (200 <= response[0] <= 299):
                     conn.helo()
@@ -474,9 +483,9 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                     conn.login(smtpuser, smtppasswd)
                 refused = conn.sendmail(fromaddr, toaddrs, message)
                 conn.quit()
-            except (socket.error, smtplib.SMTPException) as x:
+            except (socket.error, smtplib.SMTPException) as ex:
                 # If wrong password, try again...
-                if isinstance(x, smtplib.SMTPAuthenticationError):
+                if isinstance(ex, smtplib.SMTPAuthenticationError):
                     ewrite('SMTP error: authentication failed.  Try again.\n')
                     tryagain = True
                     smtppasswd = None
@@ -487,7 +496,7 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                         tryagain = False
 
                 # In case of failure, ask to retry or to save & exit
-                if ui.yes_no(f'SMTP send failure: {x}. You can retry, or save the report and exit. Do you want to retry?',
+                if ui.yes_no(f'SMTP send failure: {ex}. You can retry, or save the report and exit. Do you want to retry?',
                              'Yes, please retry.',
                              'No, save and exit.'):
                     tryagain = True
