@@ -1,7 +1,7 @@
 # urwid user interface for reportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 2006-08 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2023 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -32,7 +32,8 @@ from reportbug.exceptions import (
     NoPackage, NoBugs, NoReport, QuertBTSError,
 )
 from reportbug.urlutils import launch_browser
-from .text_ui import (
+from .text_ui import (  # noqa: F401
+    display_report,
     ewrite,
     spawn_editor,
     system
@@ -124,7 +125,7 @@ class dialog(object):
         self.view = w
 
     def add_buttons(self, buttons, default=0, vertical=False):
-        l = []
+        li = []
         for name, exitcode in buttons:
             if exitcode == '---':
                 # Separator is just a text label
@@ -134,15 +135,15 @@ class dialog(object):
                 b = urwid.Button(name, self.button_press)
                 b.exitcode = exitcode
                 b = urwid.AttrWrap(b, 'selectable', 'focus')
-            l.append(b)
+            li.append(b)
 
         if vertical:
-            box = urwid.ListBox(l)
+            box = urwid.ListBox(li)
             box.set_focus(default or 0)
             self.buttons = urwid.Frame(urwid.AttrWrap(box, 'selectable'))
-            self.frame.footer = urwid.BoxAdapter(self.buttons, min(len(l), 10))
+            self.frame.footer = urwid.BoxAdapter(self.buttons, min(len(li), 10))
         else:
-            self.buttons = urwid.GridFlow(l, 12, 3, 1, 'center')
+            self.buttons = urwid.GridFlow(li, 12, 3, 1, 'center')
             self.buttons.set_focus(default or 0)
             self.frame.footer = urwid.Pile([urwid.Divider(), self.buttons],
                                            focus_item=1)
@@ -241,17 +242,18 @@ class textentry(dialog):
 class listdialog(dialog):
     def __init__(self, text, widgets, has_default=False, width=None,
                  height=None, title='', buttonwidth=12):
-        l = []
+        li = []
         self.items = []
         for (w, label) in widgets:
+            w.others = self.items
             self.items.append(w)
             if label:
                 w = urwid.Columns([('fixed', buttonwidth, w),
                                    urwid.Text(label)], 2)
             w = urwid.AttrWrap(w, 'selectable', 'focus')
-            l.append(w)
+            li.append(w)
 
-        lb = urwid.ListBox(l)
+        lb = urwid.ListBox(li)
         lb = urwid.AttrWrap(lb, "selectable")
         dialog.__init__(self, text, height=height, width=width, body=lb,
                         title=title)
@@ -278,11 +280,11 @@ class checklistdialog(listdialog):
         if exitcode:
             return exitcode, []
 
-        l = []
+        li = []
         for i in self.items:
             if i.get_state():
-                l.append(i.get_label())
-        return exitcode, l
+                li.append(i.get_label())
+        return exitcode, li
 
 
 def display_message(message, *args, **kwargs):
@@ -323,7 +325,7 @@ def long_message(message, *args, **kwargs):
         ui = None
 
     # Rewrap the message
-    chunks = re.split('\n\n+', message)
+    chunks = re.split('\n+', message)
     chunks = [re.sub(r'\s+', ' ', x).strip() for x in chunks]
     message = '\n\n'.join(chunks).strip()
 
@@ -333,7 +335,6 @@ def long_message(message, *args, **kwargs):
 
 
 final_message = long_message
-display_report = long_message
 display_failure = long_message
 
 
@@ -360,7 +361,7 @@ def select_options(msg, ok, help=None, allow_numbers=False, nowrap=False,
 
 
 def yes_no(msg, yeshelp, nohelp, default=True, nowrap=False, ui=None):
-    box = dialog('', long_message=msg + "?", title=VERSION)
+    box = dialog('', long_message=msg, title=VERSION)
     box.add_buttons([('Yes', True), ('No', False)], default=1 - int(default))
     result = box.main(ui)
     return result
@@ -389,23 +390,35 @@ def get_multiline(prompt, options=None, title=None, force_prompt=False,
     box = textentry(prompt, multiline=True)
     box.add_buttons([("OK", 0)])
     code, text = box.main(ui)
-    l = text.split('\n')
-    return l
+    li = text.split('\n')
+    return li
 
 
 def get_password(prompt=None):
     return getpass.getpass(prompt)
 
 
+def _enable_other_checkbox(self, state, other):
+    if state and other:
+        for widget in self.others:
+            if widget.label == other:
+                widget.set_state(True)
+
+
 def menu(par, options, prompt, default=None, title=None, any_ok=False,
          order=None, extras=None, multiple=False, empty_ok=False, ui=None,
-         oklabel='Ok', cancellabel='Cancel', quitlabel=None):
+         oklabel='Ok', cancellabel='Cancel', quitlabel=None, enable_other={}):
     if not extras:
         extras = []
     else:
         extras = list(extras)
 
-    if not default:
+    if multiple:
+        if default is None:
+            default = []
+        elif isinstance(default, str):
+            default = [default]
+    elif not default:
         default = ''
 
     if title:
@@ -442,7 +455,10 @@ def menu(par, options, prompt, default=None, title=None, any_ok=False,
     options = opts
 
     if multiple:
-        widgets = [(urwid.CheckBox(option, state=(option == default)),
+        widgets = [(urwid.CheckBox(option, state=(option in default),
+                                   on_state_change=_enable_other_checkbox,
+                                   user_data=enable_other.get(option, False),
+                                   ),
                     desc or '') for (option, desc) in options]
         box = checklistdialog(par, widgets, height=('relative', 80),
                               title=title)
@@ -485,7 +501,7 @@ def menu(par, options, prompt, default=None, title=None, any_ok=False,
     if default:
         for i, opt in enumerate(options):
             if opt[0] == default:
-                focus = i
+                focus = i  # noqa: F841
                 break
 
     result, chosen = box.main(ui)
@@ -501,13 +517,15 @@ def get_filename(prompt, title=None, force_prompt=False, default=''):
                       default=default)
 
 
-def select_multiple(par, options, prompt, title=None, order=None, extras=None):
-    return menu(par, options, prompt, title=title, order=order, extras=extras,
-                multiple=True, empty_ok=False)
+def select_multiple(par, options, prompt, default=None, title=None,
+                    order=None, extras=None, enable_other={}):
+    return menu(par, options, prompt, default=default, title=title,
+                order=order, extras=extras, multiple=True, empty_ok=False,
+                enable_other=enable_other)
 
 
 # Things that are very UI dependent go here
-def show_report(number, system, mirrors,
+def show_report(number, bts, mirrors,
                 http_proxy, timeout, screen=None, queryonly=False, title='',
                 archived='no', mbox_reader_cmd=None):
     from reportbug import debbugs
@@ -516,11 +534,11 @@ def show_report(number, system, mirrors,
     if not ui:
         ui = initialize_urwid_ui()
 
-    sysinfo = debbugs.SYSTEMS[system]
+    sysinfo = debbugs.SYSTEMS[bts]
     display_message('Retrieving report #%d from %s bug tracking system...',
                     number, sysinfo['name'], title=title, ui=ui)
 
-    info = debbugs.get_report(number, timeout, system, mirrors=mirrors,
+    info = debbugs.get_report(number, timeout, bts, mirrors=mirrors,
                               http_proxy=http_proxy, archived=archived)
     if not info:
         long_message('Bug report #%d not found.', number, title=title, ui=ui)
@@ -543,7 +561,7 @@ def show_report(number, system, mirrors,
         elif r == 'm':
             return buginfo
 
-        launch_browser(debbugs.get_report_url(system, number, archived))
+        launch_browser(debbugs.get_report_url(bts, number, archived))
     return
 
 
@@ -602,7 +620,6 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
 
             buglist = []
             for (t, bugs) in hierarchy:
-                bcount = len(bugs)
                 buglist.append(('---', t))
                 buglist_tmp = {}
                 for bug in bugs:
@@ -624,11 +641,11 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
                 quitlabel = 'Quit'
 
             while True:
-                info = int(menu('Select a bug to read (and possibly report more information) or report a new bug:', buglist,
-                            '', ui=ui, title=sectitle, default=p,
-                            oklabel='Read bug',
-                            cancellabel=cancellabel,
-                            quitlabel=quitlabel))
+                info = int(menu('Select a bug to read (and possibly report more information) or report a new bug:',
+                                buglist, '', ui=ui, title=sectitle, default=p,
+                                oklabel='Read bug',
+                                cancellabel=cancellabel,
+                                quitlabel=quitlabel))
                 ui = None
                 if info < 0:
                     if info == -1:
