@@ -2,7 +2,7 @@
 # utils module - common functions for reportbug UIs
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2023 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -37,11 +37,12 @@ except ImportError as e:
 import shlex
 import email
 import email.policy
+import enum
 import socket
 import subprocess
-import apt
 import gzip
 
+import apt
 from .urlutils import open_url
 from .mailer import MUA
 
@@ -50,7 +51,7 @@ PSEUDOHEADERS = ('Package', 'Source', 'Version', 'Severity', 'File', 'Tags',
                  'Justification', 'Followup-For', 'Owner', 'User', 'Usertags',
                  'Forwarded', 'Control', 'X-Debbugs-Cc')
 # These pseudo-headers can be repeated in the report
-REPEATABLE_PSEUDOHEADERS = ['Control', ]
+REPEATABLE_PSEUDOHEADERS = ['Control', 'User', 'Usertags']
 
 MODES = {'novice': 'Offer simple prompts, bypassing technical questions.',
          'standard': 'Offer more extensive prompts, including asking about '
@@ -61,12 +62,18 @@ MODES = {'novice': 'Offer simple prompts, bypassing technical questions.',
          'expert': 'Bypass most handholding measures and preliminary triage '
                    'routines.  This mode should not be used by people unfamiliar with '
                    'Debian\'s policies and operating procedures.'}
-MODELIST = ['novice', 'standard', 'advanced', 'expert']
-for mode in MODELIST:
-    exec('MODE_%s=%d' % (mode.upper(), MODELIST.index(mode)))
-del mode
 
-# moved here since it needs the MODE_* vars to be defined
+
+class MODE(enum.IntEnum):
+    NOVICE = 0
+    STANDARD = 1
+    ADVANCED = 2
+    EXPERT = 3
+
+
+MODELIST = [m.name.lower() for m in MODE]
+
+# moved here since it needs the MODE.* vars to be defined
 from . import debbugs   # noqa: E402
 from reportbug.ui import AVAILABLE_UIS   # noqa: E402
 
@@ -86,13 +93,13 @@ fhs_directories = ['/', '/usr', '/usr/share', '/var', '/usr/X11R6',
                    '/usr/man', '/usr/doc', '/usr/bin']
 
 # A map between codenames and suites
-CODENAME2SUITE = {'wheezy': 'oldoldoldoldstable',
-                  'jessie': 'oldoldoldstable',
-                  'stretch': 'oldoldstable',
-                  'buster': 'oldstable',
-                  'bullseye': 'stable',
-                  'bookworm': 'testing',
-                  'trixie': 'next-testing',
+CODENAME2SUITE = {'jessie': 'oldoldoldoldstable',
+                  'stretch': 'oldoldoldstable',
+                  'buster': 'oldoldstable',
+                  'bullseye': 'oldstable',
+                  'bookworm': 'stable',
+                  'trixie': 'testing',
+                  'forky': 'next-testing',
                   'sid': 'unstable',
                   'experimental': 'experimental'}
 SUITE2CODENAME = dict([(suite, codename) for codename, suite in list(CODENAME2SUITE.items())])
@@ -573,7 +580,7 @@ def get_user_id(emailaddr='', realname='', charset='utf-8'):
     return email.utils.formataddr((realname, emailaddr))
 
 
-statuscache = {}
+statuscache: dict[str, tuple] = {}
 
 
 def get_package_status(package, avail=False):
@@ -599,8 +606,8 @@ def get_package_status(package, avail=False):
             representing alternative dependencies
         3.  recommended packages as a tuple containing lists, each list
             representing alternative recommends
-        4.  conffiles as a tuple containing (filename_str, md5sum_str)
-            tuples
+        4.  conffiles as a tuple containing (filename_str, md5sum_str,
+            flag_str) tuples
         5.  maintainer name and email address string
         6.  bool indicating whether the package is installed
         7.  origin (content of "Origin:" package header if available)
@@ -651,8 +658,16 @@ def get_package_status(package, avail=False):
 
     packarg = shlex.quote(package)
     if avail:
-        output = get_command_output(
-            "LC_ALL=C.UTF-8 apt-cache show %s 2>/dev/null" % packarg)
+        # Get the information on the package from apt instead of dpkg.
+        # XXX: Only call `apt-cache show $pkg` if we already know that apt
+        # knows about the package. See #716763 for why.  A better
+        # solution would be to collect all information through
+        # python-apt directly.
+        if package in _apt_cache:
+            output = get_command_output(
+                "LC_ALL=C.UTF-8 apt-cache show %s 2>/dev/null" % packarg)
+        else:
+            output = ""
     else:
         # filter through dpkg-query to automatically append arch
         # qualifier in the cases where this is needed
@@ -680,7 +695,7 @@ def get_package_status(package, avail=False):
                 confmode = False
             else:
                 # re is used to identify also conffiles with spaces in the name
-                conffiles += re.findall(r' (.+) ([0-9a-f]+).*$', line)
+                conffiles += re.findall(r' (.+) ([0-9a-f]+|newconffile)(?: (.+))?$', line)
 
         if versionre.match(line):
             (crud, pkgversion) = line.split(": ", 1)
@@ -883,13 +898,17 @@ def get_source_version(srcname):
     str or None
         the version of the source package
     """
+    versions = []
     try:
         srcrecords = apt.apt_pkg.SourceRecords()
         while srcrecords.lookup(srcname):
             if srcrecords.package == srcname:
-                return srcrecords.version
+                versions.append(srcrecords.version)
     except apt.apt_pkg.Error as e:
         print(f"Cannot look up source package: '{e}'")
+    # do not guess if there are multiple versions to choose from
+    if len(set(versions)) == 1:
+        return versions[0]
     return None
 
 
@@ -1136,29 +1155,56 @@ def get_dependency_info(package, depends, rel="depends on"):
     # columns)
     maxp = max([len(x[0]) for x in deplist])
     maxv = max([len(x[1]) for x in deplist])
-    widthp = min(maxp, 73 - maxv)
+    widthp = min(maxp, max(73 - maxv, 15))
     widthv = min(maxv, 73 - widthp)
 
+    def table_row(status, pack, version, wp=36, wv=37):
+        """line-wrap long package names and versions"""
+        info = '{:3.3} {:{wp}.{wp}}  {:.{wv}}\n'.format(status, pack, version, wp=wp, wv=wv)
+        while pack[wp:] or version[wv:]:
+            pack = pack[wp:]
+            version = version[wv:]
+            info += '    {:{wp}.{wp}}  {:.{wv}}'.format(pack, version, wp=wp, wv=wv).rstrip() + '\n'
+        return info
+
     for (pack, vers, status) in deplist:
-        # we format the string specifying to align it in a field of a given
-        # dimension (the first {width*}) but also limit its size (the second
-        # {width*}
-        info = '{0:3.3} {1:{widthp}.{widthp}}  {2:{widthv}.{widthv}}'.format(
-            status, pack, vers, widthp=widthp, widthv=widthv)
-        # remove tailing white spaces
-        depinfo += info.rstrip() + '\n'
+        depinfo += table_row(status, pack, vers, wp=widthp, wv=widthv)
 
     return depinfo
 
 
+def has_multiarch_same(src_package):
+    """Check if a source package has any binary packages with Multi-Arch: same
+
+    Parameters
+    ----------
+    src_package : str
+        the source package to find binary packages for
+
+    Returns
+    -------
+    bool
+        Indicates whether Multi-Arch: same was found (True) or not (False)
+    """
+    cache = apt.apt_pkg.Cache(None)
+    src = apt.apt_pkg.SourceRecords()
+    src.lookup(src_package)
+    for bin_package in src.binaries:
+        if bin_package in cache:
+            for version in cache[bin_package].version_list:
+                if version.multi_arch & version.MULTI_ARCH_SAME:
+                    return True
+    return False
+
+
 def get_changed_config_files(conffiles, nocompress=False):
     """
     Get information about modified config files
 
     Parameters
     ----------
-    conffiles : [(str, str), ...]
-        List of (filename, md5sum) tuples
+    conffiles : [(str, str, str), ...]
+        List of (filename, md5sum, flag) tuples
     nocompress : bool
         determines whether empty lines and comment lines should be
         included in the output
@@ -1174,11 +1220,18 @@ def get_changed_config_files(conffiles, nocompress=False):
     """
     confinfo = {}
     changed = []
-    for (filename, md5sum) in conffiles:
+    for (filename, md5sum, flag) in conffiles:
         try:
             with open(filename, errors='backslashreplace') as fp:
                 lines = fp.readlines()
+        except FileNotFoundError:
+            if flag in ("obsolete", "remove-on-upgrade"):
+                continue
+            confinfo[filename] = "[file not found]"
+            continue
         except OSError as msg:
+            if flag in ("obsolete", "remove-on-upgrade"):
+                continue
             confinfo[filename] = msg
             continue
 
@@ -1307,10 +1360,22 @@ def get_multiarch():
     return ', '.join(out.splitlines())
 
 
+def get_merged_usr():
+    """
+    Check if the system is merged-usr
+
+    Returns
+    -------
+    str
+        Empty if merged-usr, a note that the system is not merged-usr otherwise
+    """
+    return '' if os.path.realpath('/lib') == '/usr/lib' else 'merged-usr: no\n'
+
+
 def generate_blank_report(package, pkgversion, severity, justification,
                           depinfo, confinfo, foundfile='', incfiles='',
                           system='debian', exinfo=None, type=None, klass='',
-                          subject='', tags='', body='', mode=MODE_EXPERT,  # noqa: F821
+                          subject='', tags='', body='', mode=MODE.EXPERT,  # noqa: F821
                           pseudos=None, debsumsoutput=None, issource=False,
                           options=None):
     """
@@ -1852,7 +1917,7 @@ def check_package_name(pkg):
     Check the package name against Debian Policy
 
     Package names must comply with the policy requirements:
-    https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-Source
+    https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-source
 
     Returns
     -------
@@ -1935,7 +2000,8 @@ def get_kernel_taint_flags():
     -------
     list of strings
     """
-    # https://github.com/torvalds/linux/blob/cedc5b6aab493f6b1b1d381dccc0cc082da7d3d8/include/linux/kernel.h#L582
+    # https://github.com/torvalds/linux/blob/2852ca7fba9f77b204f0fe953b31fadd0057c936/include/linux/panic.h#L52
+
     # this is going to need updating (but maybe not that often)
     TAINT_FLAGS = [
         'TAINT_PROPRIETARY_MODULE',
@@ -1956,6 +2022,7 @@ def get_kernel_taint_flags():
         'TAINT_LIVEPATCH',
         'TAINT_AUX',
         'TAINT_RANDSTRUCT',
+        'TAINT_TEST',
     ]
 
     flags = []
